{#- OpenAI Codex skill wrapper template -#}
{#- Variables: name, description, invocation, title, template, related -#}
{#- NOTE: Codex requires frontmatter at the VERY START of file (no comments before ---) -#}
---
name: {{ name }}
description: "{{ description }}"
---

<!-- GENERATED FILE - DO NOT EDIT DIRECTLY -->
<!-- Source: integrations/_templates/ + _config/skills.yaml -->
<!-- Regenerate: make assemble-skills -->

# {{ title | default(name | title) }} (Codex)

{{ description }}

## Invocation

```
{{ invocation | default('$' ~ name) }}
```

Note: Codex reserves `/...` for built-in commands. Use `${{ name }}` instead.

{% include "skills/" ~ template %}

{% if related %}
---

## Related Skills

{% for r in related %}
- `${{ r }}`
{% endfor %}
{% endif %}
